#include "GSMclick_objects.h"
#include "GSMclick_resources.h"
#include "built_in.h"


// Glcd module connections
char GLCD_DataPort at PORTD;
sbit GLCD_CS1 at LATB0_bit;
sbit GLCD_CS2 at LATB1_bit;
sbit GLCD_RS at LATB2_bit;
sbit GLCD_RW at LATB3_bit;
sbit GLCD_EN at LATB4_bit;
sbit GLCD_RST at LATB5_bit;
sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction at TRISB2_bit;
sbit GLCD_RW_Direction at TRISB3_bit;
sbit GLCD_EN_Direction at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveA at LATC0_bit;
sbit DriveB at LATC1_bit;
sbit DriveA_Direction at TRISC0_bit;
sbit DriveB_Direction at TRISC1_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 900;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int Glcd_caption_length, Glcd_caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TBox *local_box;
TBox *exec_box;
int box_order;



void Init_ADC() {
  ADC_Init();
}
static void InitializeTouchPanel() {
  Init_ADC();
  Glcd_Init();                                             // Initialize GLCD
  Glcd_Fill(0);                                            // Clear GLCD

  TP_Init(128, 64, 0, 1);                                  // Initialize touch panel
  TP_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Dialer;
  TBox                   Box1;
  TButton               Button1;
char Button1_Caption[2] = "1";

  TButton               Button2;
char Button2_Caption[2] = "2";

  TButton               Button3;
char Button3_Caption[2] = "3";

  TButton               Button4;
char Button4_Caption[2] = "4";

  TButton               Button5;
char Button5_Caption[2] = "5";

  TButton               Button6;
char Button6_Caption[2] = "6";

  TButton               Button7;
char Button7_Caption[2] = "7";

  TButton               Button8;
char Button8_Caption[2] = "8";

  TButton               Button9;
char Button9_Caption[2] = "9";

  TButton               Button10;
char Button10_Caption[2] = "0";

  TButton               btnPhone;
char btnPhone_Caption[2] = ">";

  TButton               btnDel;
char btnDel_Caption[5] = " del";

  TButton               btnX;
char btnX_Caption[6] = "clear";

  TLabel                 lblNumber;
char lblNumber_Caption[21] = " ";

  TButton               ButtonPlus;
char ButtonPlus_Caption[2] = "+";

  TButton                * const code Screen1_Buttons[14]=
         {
         &Button1,             
         &Button2,             
         &Button3,             
         &Button4,             
         &Button5,             
         &Button6,             
         &Button7,             
         &Button8,             
         &Button9,             
         &Button10,            
         &btnPhone,            
         &btnDel,              
         &btnX,                
         &ButtonPlus           
         };
  TLabel                 * const code Screen1_Labels[1]=
         {
         &lblNumber            
         };
  TBox                   * const code Screen1_Boxes[1]=
         {
         &Box1                 
         };


  TScreen                Phone;
  TButton               btnCall;
char btnCall_Caption[8] = "Call";

  TButton               btnEnd;
char btnEnd_Caption[8] = "End";

  TLine                  Line1;
  TLine                  Line2;
  TLine                  Line3;
  TLine                  Line4;
  TLine                  Line5;
  TButton               btnDialer;
char btnDialer_Caption[8] = " Dialer";

  TLabel                 lblInfo;
char lblInfo_Caption[21] = "Example";

  TLabel                 lblStatus;
char lblStatus_Caption[21] = "GSM click";

  TLine                  Line6;
  TButton                * const code Screen2_Buttons[3]=
         {
         &btnCall,             
         &btnEnd,              
         &btnDialer            
         };
  TLabel                 * const code Screen2_Labels[2]=
         {
         &lblInfo,             
         &lblStatus            
         };
  TLine                  * const code Screen2_Lines[6]=
         {
         &Line1,               
         &Line2,               
         &Line3,               
         &Line4,               
         &Line5,               
         &Line6                
         };




static void InitializeObjects() {
  Dialer.ButtonsCount              = 14;
  Dialer.Buttons                   = Screen1_Buttons;
  Dialer.LabelsCount               = 1;
  Dialer.Labels                    = Screen1_Labels;
  Dialer.BoxesCount                = 1;
  Dialer.Boxes                     = Screen1_Boxes;
  Dialer.LinesCount                = 0;
  Dialer.ObjectsCount              = 16;

  Phone.ButtonsCount              = 3;
  Phone.Buttons                   = Screen2_Buttons;
  Phone.LabelsCount               = 2;
  Phone.Labels                    = Screen2_Labels;
  Phone.BoxesCount                = 0;
  Phone.LinesCount                = 6;
  Phone.Lines                     = Screen2_Lines;
  Phone.ObjectsCount              = 11;


  Box1.OwnerScreen     = &Dialer;
  Box1.Order           = 0;
  Box1.Left            = 1;
  Box1.Top             = 1;
  Box1.Width           = 108;
  Box1.Height          = 19;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.OnUpPtr         = 0;
  Box1.OnDownPtr       = 0;
  Box1.OnClickPtr      = 0;
  Box1.OnPressPtr      = 0;

  Button1.OwnerScreen     = &Dialer;
  Button1.Order           = 1;
  Button1.Left            = 1;
  Button1.Top             = 22;
  Button1.Width           = 18;
  Button1.Height          = 20;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Caption         = Button1_Caption;
  Button1.TextAlign       = _taCenter;
  Button1.FontName        = Tahoma14x16_Bold;
  Button1.PressColEnabled = 1;
  Button1.OnUpPtr         = 0;
  Button1.OnDownPtr       = 0;
  Button1.OnClickPtr      = Button1OnClick;
  Button1.OnPressPtr      = 0;

  Button2.OwnerScreen     = &Dialer;
  Button2.Order           = 2;
  Button2.Left            = 20;
  Button2.Top             = 22;
  Button2.Width           = 18;
  Button2.Height          = 20;
  Button2.Visible         = 1;
  Button2.Active          = 1;
  Button2.Caption         = Button2_Caption;
  Button2.TextAlign       = _taCenter;
  Button2.FontName        = Tahoma14x16_Bold;
  Button2.PressColEnabled = 1;
  Button2.OnUpPtr         = 0;
  Button2.OnDownPtr       = 0;
  Button2.OnClickPtr      = Button2OnClick;
  Button2.OnPressPtr      = 0;

  Button3.OwnerScreen     = &Dialer;
  Button3.Order           = 3;
  Button3.Left            = 39;
  Button3.Top             = 22;
  Button3.Width           = 18;
  Button3.Height          = 20;
  Button3.Visible         = 1;
  Button3.Active          = 1;
  Button3.Caption         = Button3_Caption;
  Button3.TextAlign       = _taCenter;
  Button3.FontName        = Tahoma14x16_Bold;
  Button3.PressColEnabled = 1;
  Button3.OnUpPtr         = 0;
  Button3.OnDownPtr       = 0;
  Button3.OnClickPtr      = Button3OnClick;
  Button3.OnPressPtr      = 0;

  Button4.OwnerScreen     = &Dialer;
  Button4.Order           = 4;
  Button4.Left            = 58;
  Button4.Top             = 22;
  Button4.Width           = 18;
  Button4.Height          = 20;
  Button4.Visible         = 1;
  Button4.Active          = 1;
  Button4.Caption         = Button4_Caption;
  Button4.TextAlign       = _taCenter;
  Button4.FontName        = Tahoma14x16_Bold;
  Button4.PressColEnabled = 1;
  Button4.OnUpPtr         = 0;
  Button4.OnDownPtr       = 0;
  Button4.OnClickPtr      = Button4OnClick;
  Button4.OnPressPtr      = 0;

  Button5.OwnerScreen     = &Dialer;
  Button5.Order           = 5;
  Button5.Left            = 77;
  Button5.Top             = 22;
  Button5.Width           = 18;
  Button5.Height          = 20;
  Button5.Visible         = 1;
  Button5.Active          = 1;
  Button5.Caption         = Button5_Caption;
  Button5.TextAlign       = _taCenter;
  Button5.FontName        = Tahoma14x16_Bold;
  Button5.PressColEnabled = 1;
  Button5.OnUpPtr         = 0;
  Button5.OnDownPtr       = 0;
  Button5.OnClickPtr      = Button5OnClick;
  Button5.OnPressPtr      = 0;

  Button6.OwnerScreen     = &Dialer;
  Button6.Order           = 6;
  Button6.Left            = 1;
  Button6.Top             = 43;
  Button6.Width           = 18;
  Button6.Height          = 20;
  Button6.Visible         = 1;
  Button6.Active          = 1;
  Button6.Caption         = Button6_Caption;
  Button6.TextAlign       = _taCenter;
  Button6.FontName        = Tahoma14x16_Bold;
  Button6.PressColEnabled = 1;
  Button6.OnUpPtr         = 0;
  Button6.OnDownPtr       = 0;
  Button6.OnClickPtr      = Button6OnClick;
  Button6.OnPressPtr      = 0;

  Button7.OwnerScreen     = &Dialer;
  Button7.Order           = 7;
  Button7.Left            = 20;
  Button7.Top             = 43;
  Button7.Width           = 18;
  Button7.Height          = 20;
  Button7.Visible         = 1;
  Button7.Active          = 1;
  Button7.Caption         = Button7_Caption;
  Button7.TextAlign       = _taCenter;
  Button7.FontName        = Tahoma14x16_Bold;
  Button7.PressColEnabled = 1;
  Button7.OnUpPtr         = 0;
  Button7.OnDownPtr       = 0;
  Button7.OnClickPtr      = Button7OnClick;
  Button7.OnPressPtr      = 0;

  Button8.OwnerScreen     = &Dialer;
  Button8.Order           = 8;
  Button8.Left            = 39;
  Button8.Top             = 43;
  Button8.Width           = 18;
  Button8.Height          = 20;
  Button8.Visible         = 1;
  Button8.Active          = 1;
  Button8.Caption         = Button8_Caption;
  Button8.TextAlign       = _taCenter;
  Button8.FontName        = Tahoma14x16_Bold;
  Button8.PressColEnabled = 1;
  Button8.OnUpPtr         = 0;
  Button8.OnDownPtr       = 0;
  Button8.OnClickPtr      = Button8OnClick;
  Button8.OnPressPtr      = 0;

  Button9.OwnerScreen     = &Dialer;
  Button9.Order           = 9;
  Button9.Left            = 58;
  Button9.Top             = 43;
  Button9.Width           = 18;
  Button9.Height          = 20;
  Button9.Visible         = 1;
  Button9.Active          = 1;
  Button9.Caption         = Button9_Caption;
  Button9.TextAlign       = _taCenter;
  Button9.FontName        = Tahoma14x16_Bold;
  Button9.PressColEnabled = 1;
  Button9.OnUpPtr         = 0;
  Button9.OnDownPtr       = 0;
  Button9.OnClickPtr      = Button9OnClick;
  Button9.OnPressPtr      = 0;

  Button10.OwnerScreen     = &Dialer;
  Button10.Order           = 10;
  Button10.Left            = 77;
  Button10.Top             = 43;
  Button10.Width           = 18;
  Button10.Height          = 20;
  Button10.Visible         = 1;
  Button10.Active          = 1;
  Button10.Caption         = Button10_Caption;
  Button10.TextAlign       = _taCenter;
  Button10.FontName        = Tahoma14x16_Bold;
  Button10.PressColEnabled = 1;
  Button10.OnUpPtr         = 0;
  Button10.OnDownPtr       = 0;
  Button10.OnClickPtr      = Button10OnClick;
  Button10.OnPressPtr      = 0;

  btnPhone.OwnerScreen     = &Dialer;
  btnPhone.Order           = 11;
  btnPhone.Left            = 110;
  btnPhone.Top             = 1;
  btnPhone.Width           = 17;
  btnPhone.Height          = 19;
  btnPhone.Visible         = 1;
  btnPhone.Active          = 1;
  btnPhone.Caption         = btnPhone_Caption;
  btnPhone.TextAlign       = _taCenter;
  btnPhone.FontName        = Glcd_defaultFont;
  btnPhone.PressColEnabled = 1;
  btnPhone.OnUpPtr         = 0;
  btnPhone.OnDownPtr       = 0;
  btnPhone.OnClickPtr      = btnPhoneOnClick;
  btnPhone.OnPressPtr      = 0;

  btnDel.OwnerScreen     = &Dialer;
  btnDel.Order           = 12;
  btnDel.Left            = 110;
  btnDel.Top             = 43;
  btnDel.Width           = 17;
  btnDel.Height          = 20;
  btnDel.Visible         = 1;
  btnDel.Active          = 1;
  btnDel.Caption         = btnDel_Caption;
  btnDel.TextAlign       = _taCenter;
  btnDel.FontName        = Glcd_defaultFont;
  btnDel.PressColEnabled = 1;
  btnDel.OnUpPtr         = 0;
  btnDel.OnDownPtr       = 0;
  btnDel.OnClickPtr      = btnDelOnClick;
  btnDel.OnPressPtr      = 0;

  btnX.OwnerScreen     = &Dialer;
  btnX.Order           = 13;
  btnX.Left            = 96;
  btnX.Top             = 22;
  btnX.Width           = 31;
  btnX.Height          = 20;
  btnX.Visible         = 1;
  btnX.Active          = 1;
  btnX.Caption         = btnX_Caption;
  btnX.TextAlign       = _taCenter;
  btnX.FontName        = Glcd_defaultFont;
  btnX.PressColEnabled = 1;
  btnX.OnUpPtr         = 0;
  btnX.OnDownPtr       = 0;
  btnX.OnClickPtr      = btnXOnClick;
  btnX.OnPressPtr      = 0;

  lblNumber.OwnerScreen     = &Dialer;
  lblNumber.Order           = 14;
  lblNumber.Left            = 2;
  lblNumber.Top             = 2;
  lblNumber.Width           = 0;
  lblNumber.Height          = 17;
  lblNumber.Visible         = 1;
  lblNumber.Active          = 1;
  lblNumber.Caption         = lblNumber_Caption;
  lblNumber.FontName        = Tahoma13x16_Regular;
  lblNumber.Font_Color      = 1;
  lblNumber.OnUpPtr         = 0;
  lblNumber.OnDownPtr       = 0;
  lblNumber.OnClickPtr      = 0;
  lblNumber.OnPressPtr      = 0;

  ButtonPlus.OwnerScreen     = &Dialer;
  ButtonPlus.Order           = 15;
  ButtonPlus.Left            = 96;
  ButtonPlus.Top             = 43;
  ButtonPlus.Width           = 13;
  ButtonPlus.Height          = 20;
  ButtonPlus.Visible         = 1;
  ButtonPlus.Active          = 1;
  ButtonPlus.Caption         = ButtonPlus_Caption;
  ButtonPlus.TextAlign       = _taCenter;
  ButtonPlus.FontName        = Tahoma12x11_Bold;
  ButtonPlus.PressColEnabled = 1;
  ButtonPlus.OnUpPtr         = 0;
  ButtonPlus.OnDownPtr       = 0;
  ButtonPlus.OnClickPtr      = ButtonPlusOnClick;
  ButtonPlus.OnPressPtr      = 0;

  btnCall.OwnerScreen     = &Phone;
  btnCall.Order           = 0;
  btnCall.Left            = 1;
  btnCall.Top             = 43;
  btnCall.Width           = 62;
  btnCall.Height          = 20;
  btnCall.Visible         = 1;
  btnCall.Active          = 0;
  btnCall.Caption         = btnCall_Caption;
  btnCall.TextAlign       = _taCenter;
  btnCall.FontName        = Tahoma13x16_Regular;
  btnCall.PressColEnabled = 1;
  btnCall.OnUpPtr         = 0;
  btnCall.OnDownPtr       = 0;
  btnCall.OnClickPtr      = btnCallOnClick;
  btnCall.OnPressPtr      = 0;

  btnEnd.OwnerScreen     = &Phone;
  btnEnd.Order           = 1;
  btnEnd.Left            = 64;
  btnEnd.Top             = 43;
  btnEnd.Width           = 63;
  btnEnd.Height          = 20;
  btnEnd.Visible         = 1;
  btnEnd.Active          = 0;
  btnEnd.Caption         = btnEnd_Caption;
  btnEnd.TextAlign       = _taCenter;
  btnEnd.FontName        = Tahoma13x16_Regular;
  btnEnd.PressColEnabled = 1;
  btnEnd.OnUpPtr         = 0;
  btnEnd.OnDownPtr       = 0;
  btnEnd.OnClickPtr      = btnEndOnClick;
  btnEnd.OnPressPtr      = 0;

  Line1.OwnerScreen     = &Phone;
  Line1.Order           = 2;
  Line1.First_Point_X   = 1;
  Line1.First_Point_Y   = 1;
  Line1.Second_Point_X  = 1;
  Line1.Second_Point_Y  = 41;
  Line1.Visible         = 1;

  Line2.OwnerScreen     = &Phone;
  Line2.Order           = 3;
  Line2.First_Point_X   = 1;
  Line2.First_Point_Y   = 1;
  Line2.Second_Point_X  = 95;
  Line2.Second_Point_Y  = 1;
  Line2.Visible         = 1;

  Line3.OwnerScreen     = &Phone;
  Line3.Order           = 4;
  Line3.First_Point_X   = 95;
  Line3.First_Point_Y   = 1;
  Line3.Second_Point_X  = 95;
  Line3.Second_Point_Y  = 18;
  Line3.Visible         = 1;

  Line4.OwnerScreen     = &Phone;
  Line4.Order           = 5;
  Line4.First_Point_X   = 126;
  Line4.First_Point_Y   = 41;
  Line4.Second_Point_X  = 126;
  Line4.Second_Point_Y  = 18;
  Line4.Visible         = 1;

  Line5.OwnerScreen     = &Phone;
  Line5.Order           = 6;
  Line5.First_Point_X   = 95;
  Line5.First_Point_Y   = 18;
  Line5.Second_Point_X  = 126;
  Line5.Second_Point_Y  = 18;
  Line5.Visible         = 1;

  btnDialer.OwnerScreen     = &Phone;
  btnDialer.Order           = 7;
  btnDialer.Left            = 97;
  btnDialer.Top             = 1;
  btnDialer.Width           = 30;
  btnDialer.Height          = 16;
  btnDialer.Visible         = 1;
  btnDialer.Active          = 1;
  btnDialer.Caption         = btnDialer_Caption;
  btnDialer.TextAlign       = _taCenter;
  btnDialer.FontName        = Glcd_defaultFont;
  btnDialer.PressColEnabled = 1;
  btnDialer.OnUpPtr         = 0;
  btnDialer.OnDownPtr       = 0;
  btnDialer.OnClickPtr      = btnDialerOnClick;
  btnDialer.OnPressPtr      = 0;

  lblInfo.OwnerScreen     = &Phone;
  lblInfo.Order           = 8;
  lblInfo.Left            = 4;
  lblInfo.Top             = 21;
  lblInfo.Width           = 50;
  lblInfo.Height          = 17;
  lblInfo.Visible         = 1;
  lblInfo.Active          = 1;
  lblInfo.Caption         = lblInfo_Caption;
  lblInfo.FontName        = Tahoma13x16_Regular;
  lblInfo.Font_Color      = 1;
  lblInfo.OnUpPtr         = 0;
  lblInfo.OnDownPtr       = 0;
  lblInfo.OnClickPtr      = 0;
  lblInfo.OnPressPtr      = 0;

  lblStatus.OwnerScreen     = &Phone;
  lblStatus.Order           = 9;
  lblStatus.Left            = 3;
  lblStatus.Top             = 3;
  lblStatus.Width           = 64;
  lblStatus.Height          = 16;
  lblStatus.Visible         = 1;
  lblStatus.Active          = 1;
  lblStatus.Caption         = lblStatus_Caption;
  lblStatus.FontName        = Arial13x16_Bold;
  lblStatus.Font_Color      = 1;
  lblStatus.OnUpPtr         = 0;
  lblStatus.OnDownPtr       = 0;
  lblStatus.OnClickPtr      = 0;
  lblStatus.OnPressPtr      = 0;

  Line6.OwnerScreen     = &Phone;
  Line6.Order           = 10;
  Line6.First_Point_X   = 1;
  Line6.First_Point_Y   = 41;
  Line6.Second_Point_X  = 126;
  Line6.Second_Point_Y  = 41;
  Line6.Visible         = 1;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      Glcd_Rectangle(Abutton->Left - 1, Abutton->Top - 1,
                     Abutton->Left + Abutton->Width,
                     Abutton->Top + Abutton->Height, _clDraw);
      Glcd_Rectangle(Abutton->Left, Abutton->Top,
                     Abutton->Left + Abutton->Width - 1,
                     Abutton->Top + Abutton->Height - 1, _clClear);
      Glcd_Box(Abutton->Left + 1, Abutton->Top + 1,
                     Abutton->Left + Abutton->Width - 2,
                     Abutton->Top + Abutton->Height - 2, _clDraw);
    }
    else {
      Glcd_Rectangle(Abutton->Left - 1, Abutton->Top - 1,
                    Abutton->Left + Abutton->Width,
                    Abutton->Top + Abutton->Height, _clClear);
      Glcd_Box(Abutton->Left, Abutton->Top,
        Abutton->Left + Abutton->Width - 1,
        Abutton->Top + Abutton->Height - 1, _clDraw);
      if ((Abutton->Width >= 5) && (Abutton->Height >= 5)) {
        Glcd_Rectangle(Abutton->Left + 1, Abutton->Top + 1,
          Abutton->Left + Abutton->Width - 2,
          Abutton->Top + Abutton->Height - 2, _clClear);
      }
    }
    Glcd_Set_Font_Adv(Abutton->FontName, _clClear, _GLCD_HORIZONTAL);
    Glcd_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      Glcd_Write_Text_Adv(Abutton->Caption, Abutton->Left + 3, (Abutton->Top + ((Abutton->Height - Glcd_caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      Glcd_Write_Text_Adv(Abutton->Caption, (Abutton->Left + (Abutton->Width - Glcd_caption_length) / 2), (Abutton->Top + ((Abutton->Height - Glcd_caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      Glcd_Write_Text_Adv(Abutton->Caption, Abutton->Left + (Abutton->Width - Glcd_caption_length - 4), (Abutton->Top + ((Abutton->Height - Glcd_caption_height) / 2)));
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    Glcd_Set_Font_Adv(ALabel->FontName, ALabel->Font_Color, _GLCD_HORIZONTAL);
    Glcd_Write_Text_Adv(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (ABox->Transparent == 1) {
      Glcd_Rectangle(ABox->Left, ABox->Top,
        ABox->Left + ABox->Width - 1,
        ABox->Top + ABox->Height - 1, _clDraw);
    }
    else {
      Glcd_Box(ABox->Left, ABox->Top,
        ABox->Left + ABox->Width - 1,
        ABox->Top + ABox->Height - 1, _clDraw);
    }
  }
}

void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    Glcd_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y, _clDraw);
  }
}

void DrawScreen(TScreen *aScreen) {
 int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short line_idx;
  TLine *local_line;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  box_idx = 0;
  line_idx = 0;
  CurrentScreen = aScreen;

  Glcd_Fill(0x00);

  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (line_idx  < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  button_order        = -1;
  label_order         = -1;
  box_order           = -1;
  //  Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->ButtonsCount ; _object_count++ ) {
    local_button = GetButton(_object_count);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  _object_count = -1;
  if (button_order > _object_count)
    _object_count = button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (box_order >  _object_count )
    _object_count = box_order;
}


static void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_button         = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if ((exec_button->PressColEnabled == 1) && (exec_button->OwnerScreen == CurrentScreen)) {
          DrawButton(exec_button);
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_button         = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawButton(exec_button);
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  ANSELA = 3; // Configure AN0 and AN1 pins as analog
  ANSELB = 0; // Configure PORTB pins as digital
  ANSELC = 0; // Configure PORTC pins as digital
  ANSELD = 0; // Configure PORTD pins as digital
  TRISA  = 3; // Configure AN0 and AN1 pins as input
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_Set_Calibration_Consts(39, 974, 85, 931);    // Set calibration constants

  InitializeObjects();
  DrawScreen(&Phone);
}
