/*
 * Project name:
      GSM click
 * Copyright:
      (c) mikroElektronika, 2012.
 * Revision History:
      20121225:
      - Initial release (DO);
 * Description:
      This is a demonstration how GSM click can be used to establish calls.
      The user iterface was created with help of VisualTFT software.
      It contain the two main displays, phone/info panel, and dialer panel.
      Dialer panel enables you to dial desired number and call it.

      Beside dialing the desired number example project enables you to receive the
      incoming calls, showing the caller ID/number.
      If you miss the call you can call the number of last caller.

      On board Buzzer is used to signalize the incomin calls.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     Ext. Modules:    GSM click - ac:GSM_click
                      http://www.mikroe.com/click/gsm/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
      - Place GSM click board in the mikroBUS socket 1.
      - Connect GLCD and Touch Panle
      - Place Buzzer Jumper in RC2 position
      - Swithc ON SW3.5, SW3.6, SW3.7, SW3.7 for activating the Touchpanel
      - Switch ON SW4.6 for GLCD backlight - Adjust contrast with P3
*/

#include "GSMclick_objects.h"
#include "GSMclick_user_code.h"
#include "GSMclick_resources.h"
/* set of AT commands */
// Initialiyation commands
char atc1[] = "ATE0";                      // disable command echo
char atc2[] = "AT#HFMICG=4";               // handsfree microphone gain
char atc3[] = "AT#SHFEC=1";                // handsfree echo canceller
char atc4[] = "AT+CLVL=12";                // loudspeaker volume level
char atc5[] = "AT#SRS= 3,0";               // select ringer sound
char atc6[] = "ATS0=0";                    // number of rings to auto answer (auto answer disabled)
char atc7[] = "AT+CRC=1";                  // Enable extended format reporting
char atc8[] = "AT+CLIP=1";                 // Enable caller number identification
char atc9[] = "AT#QSS?";                   // Query SIM Status
// Functional commands
char atcATS1[] = "ATS1?";                   // Get RINGING counter
char atcATD [] = "ATD";                     // Dial a phone number ATDnumber;
char atcATH [] = "ATH";                     // Hang up
char atcATA [] = "ATA";                     // Answer a call

char number[14] = {0};                      // Number which will be sent appended to ATD command
// GLCD interface messages
const MESSAGE_LENGTH = 16;
char msgInit[]      = "Initializing...";
char msgSet[]       = "Ready";
char msgDial[]      = "Dial  a  number";
char msgNoSIM[]     = "NO SIM CARD";
char msgInsertSIM[] = "Insert SIM card";
char msgLastNUM[]   = "Dialed num.:";
char msgCallFrom[]  = "Call from...";
char msgMissed[]    = "Missed call:";
// Button captions
char btnCap_Answer[] = "Answer";
char btnCap_Reject[] = "Reject";
char btnCap_Call[]   = "Call";
char btnCap_End[]    = "End";

// responses to parse
const GSM_OK    = 0;
const GSM_RING  = 1;
const GSM_NC    = 2;
const GSM_DROP  = 3;

char gsm_state = 0;
char response_rcvd = 0;
short responseID = -1, response = -1;
char msg[MESSAGE_LENGTH+1];

char phoneScreen = 1;
char receiveCall = 0;
char callerid = 0;
char incall = 0;

char cid_counter;
char dummy = 1;
char SIM_ACTIVE = 0;


//---------------------END of VAR Declaration

void interrupt(){
char tmp;

  if(RC1IF_bit == 1) {
    tmp = UART1_Read();                           // get received byte
    // process reception through state machine
    // we are parsing only "OK" , "+CLIP", "NO CARRIER", "#QSS" and RING counter "000" responses
    switch (gsm_state) {
      case  0:
        response = -1;                   // clear response
        if (tmp == 'O')                  // we have 'O', it could be "OK"
          gsm_state = 1;                 // expecting 'K'
        if (tmp == 'R')                  // we have 'R', it could be "NO CARRIER"
          gsm_state = 5;                 // expecting 'I'
        if (tmp == '+')                  // Seraching for caller ID  "+CLIP"
          gsm_state = 10;
        if (tmp == '0')                  // It could be number of RING COUNTER 000 if there is no ring
          gsm_state = 15;
        if (tmp == '#')                  // It could be SIM Status "#QSS"
          gsm_state = 20;
      break;

      case  1:
        if (tmp == 'K') {                // we have 'K' ->
          response = GSM_OK;             // we have "OK" response
          gsm_state = 50;                // expecting CR+LF
        }
        else
          gsm_state = 0;                 // reset state machine
      break;

      case 5:
        if (tmp == 'R')                  // we have 'RR', it could be "NO CARRIER"
          break;                         // expecting 'I'
        else if (tmp == 'I')             // we have 'RRI' it could be "NO CARRIER"
          break;                         // expecting 'E'
        else if (tmp == 'E')             // we have 'RRIE' it could be "NO CARRIER"
          gsm_state = 6;
        else
          gsm_state = 0;                 // reset state machine
      break;
      
      case 6:
        if (tmp == 'R') {                // we have 'R' ->
          response = GSM_NC;             // we have "NO CARRIER" response
          gsm_state = 50;                // expecting CR+LF
        }
        else
          gsm_state = 0;                 // reset state machine
      break;
      
      case 10:
        if (tmp == 'C')                  // we have '+C' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == 'L')              // we have '+CL' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == 'I')              // we have '+CLI' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == 'P')              // we have '+CLIP' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == ':')              // we have '+CLIP:' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == ' ')              // we have '+CLIP: ' ->
          break;                         // we could have "+CLIP" response
        else if(tmp == '"'){             // we have '+CLIP: "' ->
          gsm_state = 11;                // we have "+CLIP: " now comes caller number
          number[0] = 0;                 // reset number variable
          cid_counter = 0;               // reset id_number counter
        }
        else
          gsm_state = 0;                 // reset state machine
      break;
      case 11:                           // receving caller number
        if(tmp == '"'){                  // we have recived " which means we recived complete caller number
          number[cid_counter] = 0;       // add zero terminating character to number string
          response = GSM_RING;           // if we have the caller number,  then we have incoming call
          dummy = 1;                     // reset counter which will check is activiti of incoming call
          gsm_state = 12;                // The resto of the informations are not importat take a break
        }
        else{
          number[cid_counter++] = tmp;
        }
      break;
      case 12:                           // Break state - waiting for <CR>
        if(tmp == 13){
          gsm_state = 51;                // we received <CR> it's probably end of response
        }
      break;
      case 15:                           // we are decoding RING COUNTER
        if(tmp == '0'){                  // we recived another 0 now we have 00
          gsm_state = 16;
        }
        else{
          gsm_state = 0;                 // if we revice anything else after 0 - it still RINGING
        }
      break;
      case 16:
        if(tmp == '0'){
          response = GSM_DROP;
        }
        gsm_state = 50;
      break;
      case 20:
        if(tmp == 'Q')                   // we have '#Q' ->
          break;                         // we could have SIM Status
        else if (tmp == 'S')             // we have '#QSS' ->
          break;                         // we could have SIM Status
        else if (tmp == ':')             // we have '#QSS:' ->
          break;                         // we could have SIM Status
        else if (tmp == ' ')             // we have '#QSS: ' ->
          break;                         // we could have SIM Status
        else if (tmp == '0')             // we have '#QSS: 0' ->
          break;                         // we could have SIM Status
        else if (tmp == '1')             // we have '#QSS: 1' ->
          break;                         // we could have SIM Status
        else if (tmp == ',')             // we have '#QSS: x,"' ->
          gsm_state = 21;                // now comes SIM status number
        else                             //
          gsm_state = 0;                 // reset state machine counter
      break;
      case 21:
        SIM_ACTIVE = tmp - 48;
        gsm_state = 50;
      break;
      case 50:
        if (tmp == 13)                   // we have 13, it could be CR+LF
          gsm_state = 51;                // expecting LF
        else
          gsm_state = 0;                 // reset state machine
      break;
      case 51:
        if (tmp == 10) {                 // we have LF, response is complete
          response_rcvd = 1;             // set reception flag
          responseID = response;         // set response ID
        }
        gsm_state = 0;                   // reset state machine
      break;
      default:                           // unwanted character
        gsm_state = 0;                   // reset state machine
      break;
    }
  }
}

// send ATC command
void send_atc(char *s)
{
// send command string
   while(*s) {
      UART1_Write(*s++);
   }
// terminate command with CR
   UART1_Write(0x0D);
}

// get GSM response, if there is any
short get_response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return -1;
}

// wait for GSM response
void wait_response(char rspns) {
  while (get_response() != rspns)
    ;
}

// pause 1sec
void wait() {
  Delay_ms(1000);
}
void Tone1(){
Sound_Play(1480,30);
}
void Tone2(){
  Sound_Play(1330,30);
}
void Pause(){
  Delay_ms(20);
}
void Ringer(){
  Tone1();  Tone2();  Tone1();  Tone2();  Tone1();  Tone2();
  Pause();
  Tone1();  Tone2();  Tone1();  Tone2();  Tone1();  Tone2();
  Pause();
  Tone1();  Tone2();  Tone1();  Tone2();  Tone1();  Tone2();  Tone1();  Tone2();
}

void SetupGSM(){
   // disable command echo
  send_atc(atc1);
  wait_response(GSM_OK);
  // handsfree microphone gain
  send_atc(atc2);
  wait_response(GSM_OK);

  // handsfree echo canceller
  send_atc(atc3);
  wait_response(GSM_OK);

  // loudspeaker volume level
  send_atc(atc4);
  wait_response(GSM_OK);

  // select ringer sound
  send_atc(atc5);
  wait_response(GSM_OK);

  // number of rings to auto answer (auto answer disabled)
  send_atc(atc6);
  wait_response(GSM_OK);
  // Enable extended format reporting
  send_atc(atc7);
  wait_response(GSM_OK);
  // Enable caller number identification
  send_atc(atc8);
  wait_response(GSM_OK);
}

void main() {
  UART1_Init(9600);                   // Initialize UART module
  Start_TP();                         // Initialize Touch Panel
  // UART interrupt                  
  RC1IE_bit = 1;                      // Enable UART1 RX Interrupt
  RC1IF_bit = 0;                      // Reset RX Interrupt flag
  PEIE_bit  = 1;                      // Enable Pheripherial Interrupt
  GIE_bit   = 1;                      // Enable GLOBAL Interrupts
  // Buzzer Init
  Sound_Init(&PORTC, 2);              // Initialize Buzzer 
  UpdateLable(&lblInfo, &msgInit);    // Set INFO label to "Initializing..."
  Wait();  Wait();  Wait();           // Wait for module powerUP
  
  // negotiate baud rate
  while(1) {
    send_atc("AT");                   // send "AT" string until gsm sets up its baud rade
    Delay_ms(100);                    // and gets it correctly
    if (get_response() == GSM_OK)     // if gsm says "OK" on our baud rate we got it
      break;
  }
  SetupGSM();                         // Inicial set up of GSM module
  // Query GSM module for SIM card status
  send_atc(atc9);                     // Ask the module for SIM status        
  wait_response(GSM_OK);              // Wait for SIM report and OK reply
  
  if (!SIM_ACTIVE){
      UpdateLable(&lblStatus, &msgNoSIM);   // Show ERROR message
      UpdateLable(&lblInfo, &msgInsertSIM); //  both on Status and Info lable
  }
  else{
    UpdateLable(&lblInfo, &msgSet);
    Wait();
    UpdateLable(&lblInfo, &msgDial);
    while (1) {
      Check_TP();
      if (responseID == GSM_NC){             // NO CARRIER received, reset display
        response = -1;                       // Reset response flag
        responseID = -1;                     // Reset response flag
        incall = 0;                          //  set control flag
        btnCall.Caption = btnCap_Call;
        EnableButton(&btnCall);
        DrawButton(&btnCall);
        DisableButton(&btnEnd);
        DrawButton(&btnEnd);
        UpdateLable(&lblStatus, &msgLastNUM);
        btnDialer.Active = 1;                // Enable Dialer button  
      }
      if (responseID == GSM_RING){
        if(!receiveCall){                    // Phone call detedced
          btnDialer.Active = 0;              // Disable Dialer button while call is detected
          callerid = 1;                      // set control flag
          btnCall.Caption = btnCap_Answer;   // Change button captions
          btnEnd.Caption = btnCap_Reject;    //  Call - ANSWER; End - Reject
          if (!PhoneScreen){                 
            EnableButton(&btnCall);          // Dialer Screen is asctive
            EnableButton(&btnEnd);           // Update buttons
            DrawScreen(&Phone);              // Draw phone main screen
          }                                  
          else{                              
            EnableButton(&btnCall);          // Phone main screen already active 
            DrawButton(&btnCall);            //
            EnableButton(&btnEnd);           // Update buttons;
            DrawButton(&btnEnd);             //
          }                                  
          receivecall = 1;                   // set control flag
          ClearLabel(&lblInfo);              
          strcpy(lblInfo_Caption, number);
          UpdateLable(&lblStatus, &msgCallFrom);
          UpdateLable(&lblInfo, &lblInfo_Caption);
          
        }
        response = -1;                       // Reset response flag
        responseID = -1;                     // Reset response flag
      }
      if (responseID == GSM_DROP){
        btnDialer.Active = 1;                // Set up Missed call display
        receiveCall = 0;                     // Reset contol variable
        responseID = -1;                     // Reset response flag
        btnCall.Caption = btnCap_Call;       //
        btnEnd.Caption =  btnCap_End ;       // Upadate button captions
        EnableButton(&btnCall);              // Configure button
        DrawButton(&btnCall);                //
        DisableButton(&btnEnd);              // Configure button
        DrawButton(&btnEnd);                 //
        UpdateLable(&lblStatus, &msgMissed);
      }
      // Do I still need to ring?
      if (receiveCall == 1){                // Create delay between rings
        if(!dummy++){                       //  by overloading the byte variable counter
          Ringer();                         // Activate the buzzer
          send_atc(atcATS1);                // Check is the phone still RINGING
        }                                  
        Delay_ms(5);                        // Delay constant 
      }                                     
    }                                      
  }
}